# ┌─────────────────────────────────────────────────────────────────────────────────────
# │ GENERAL IMPORTS
# └─────────────────────────────────────────────────────────────────────────────────────

from typing import Generic, TypeVar

# ┌─────────────────────────────────────────────────────────────────────────────────────
# │ PROJECT IMPORTS
# └─────────────────────────────────────────────────────────────────────────────────────

from doji.core.asset.classes.asset import Asset
from doji.core.utils.classes.instance import Instance


# ┌─────────────────────────────────────────────────────────────────────────────────────
# │ ASSET PAIR
# └─────────────────────────────────────────────────────────────────────────────────────

T = TypeVar("T", bound=Asset)


class AssetPair(Generic[T], Instance):
    """A utility class that represents asset pairs"""

    # ┌─────────────────────────────────────────────────────────────────────────────────
    # │ CLASS ATTRIBUTES
    # └─────────────────────────────────────────────────────────────────────────────────

    # Declare type of base and quote
    base: T
    quote: T

    # ┌─────────────────────────────────────────────────────────────────────────────────
    # │ __STR__
    # └─────────────────────────────────────────────────────────────────────────────────

    def __str__(self) -> str:
        """String Method"""

        # Return symbol
        return self.symbol

    # ┌─────────────────────────────────────────────────────────────────────────────────
    # │ SYMBOL
    # └─────────────────────────────────────────────────────────────────────────────────

    @property
    def symbol(self) -> str:
        """Returns the symbol of the asset pair"""

        # Return the asset pair symbol
        return f"{self.base.code}-{self.quote.code}"
